@@ -32,8 +32,11 @@
 #include "check_data.h"
 #include "logger.h"
 #include "main.h"
+#include "check_parser.h"
 
 
+static tracked_file_monitor_t *current_tfile;
+
 static void
 free_file_check(checker_t *checker)
 {
@@ -53,46 +56,31 @@ dump_file_check(FILE *fp, const checker_t *checker)
 static void
 track_file_handler(const vector_t *strvec)
 {
-	virtual_server_t *vs = list_last_entry(&check_data->vs, virtual_server_t, e_list);
-	real_server_t *rs = list_last_entry(&vs->rs, real_server_t, e_list);
-	tracked_file_monitor_t *tfile;
 	tracked_file_t *vsf;
 
-	tfile = list_last_entry(&rs->track_files, tracked_file_monitor_t, e_list);
-
 	vsf = find_tracked_file_by_name(strvec_slot(strvec, 1), &check_data->track_files);
 	if (!vsf) {
 		report_config_error(CONFIG_GENERAL_ERROR, "track_file %s not found", strvec_slot(strvec, 1));
 		return;
 	}
 
-	tfile->file = vsf;
+	current_tfile->file = vsf;
 }
 
 static void
 file_check_handler(__attribute__((unused)) const vector_t *strvec)
 {
-	virtual_server_t *vs = list_last_entry(&check_data->vs, virtual_server_t, e_list);
-	real_server_t *rs = list_last_entry(&vs->rs, real_server_t, e_list);
-	tracked_file_monitor_t *tfile;
-
-	PMALLOC(tfile);
-	tfile->weight = IPVS_WEIGHT_FAULT;
-	INIT_LIST_HEAD(&tfile->e_list);
-	list_add_tail(&tfile->e_list, &rs->track_files);
+	PMALLOC(current_tfile);
+	current_tfile->weight = IPVS_WEIGHT_FAULT;
+	INIT_LIST_HEAD(&current_tfile->e_list);
 }
 
 static void
 track_file_weight_handler(const vector_t *strvec)
 {
-	virtual_server_t *vs = list_last_entry(&check_data->vs, virtual_server_t, e_list);
-	real_server_t *rs = list_last_entry(&vs->rs, real_server_t, e_list);
-	tracked_file_monitor_t *tfile;
 	int weight;
 	bool reverse = false;
 
-	tfile = list_last_entry(&rs->track_files, tracked_file_monitor_t, e_list);
-
 	if (vector_size(strvec) < 2) {
 		report_config_error(CONFIG_GENERAL_ERROR, "track file weight missing");
 		return;
@@ -116,40 +104,40 @@ track_file_weight_handler(const vector_t *strvec)
 		}
 	}
 
-	tfile->weight = weight;
-	tfile->weight_reverse = reverse;
+	current_tfile->weight = weight;
+	current_tfile->weight_reverse = reverse;
 }
 
 static void
 file_end_handler(void)
 {
-	virtual_server_t *vs = list_last_entry(&check_data->vs, virtual_server_t, e_list);
-	real_server_t *rs = list_last_entry(&vs->rs, real_server_t, e_list);
-	tracked_file_monitor_t *tfile;
-
-	tfile = list_last_entry(&rs->track_files, tracked_file_monitor_t, e_list);
-
-	if (!tfile->file) {
+	if (!current_tfile->file) {
 		report_config_error(CONFIG_GENERAL_ERROR, "FILE_CHECK has no track_file specified - ignoring");
-		free_track_file_monitor(tfile);
+		free_track_file_monitor(current_tfile);
 		return;
 	}
 
-	if (tfile->weight == IPVS_WEIGHT_FAULT) {
-		tfile->weight = tfile->file->weight;
-		tfile->weight_reverse = tfile->file->weight_reverse;
+	if (current_tfile->weight == IPVS_WEIGHT_FAULT) {
+		current_tfile->weight = current_tfile->file->weight;
+		current_tfile->weight_reverse = current_tfile->file->weight_reverse;
 	}
+
+	list_add_tail(&current_tfile->e_list, &current_rs->track_files);
+
+	current_tfile = NULL;
 }
 
 void
 install_file_check_keyword(void)
 {
+	vpp_t check_ptr;
+
 	install_keyword("FILE_CHECK", &file_check_handler);
-	install_sublevel();
+	check_ptr = install_sublevel(VPP &current_tfile);
 	install_keyword("track_file", &track_file_handler);
 	install_keyword("weight", &track_file_weight_handler);
-	install_sublevel_end_handler(&file_end_handler);
-	install_sublevel_end();
+	install_level_end_handler(&file_end_handler);
+	install_sublevel_end(check_ptr);
 }
 
 static const checker_funcs_t file_checker_funcs = { CHECKER_FILE, free_file_check, dump_file_check, NULL, NULL };
@@ -160,25 +148,24 @@ add_rs_to_track_files(void)
 	virtual_server_t *vs;
 	real_server_t *rs;
 	tracked_file_monitor_t *tfl;
-	checker_t *new_checker;
 
 	list_for_each_entry(vs, &check_data->vs, e_list) {
 		list_for_each_entry(rs, &vs->rs, e_list) {
 			list_for_each_entry(tfl, &rs->track_files, e_list) {
 				/* queue new checker - we don't have a compare function since we don't
 				 * update file checkers that way on a reload. */
-				new_checker = queue_checker(&file_checker_funcs, NULL, tfl->file, NULL, false);
-				new_checker->vs = vs;
-				new_checker->rs = rs;
+				queue_checker(&file_checker_funcs, NULL, tfl->file, NULL, false);
+				current_checker->vs = vs;
+				current_checker->rs = rs;
 
 				/* There is no concept of the checker running, but we will have
 				 * checked the file, so mark it as run. */
-				new_checker->has_run = true;
+				current_checker->has_run = true;
 
 				/* Clear Alpha mode - we know the state of the checker immediately */
-				new_checker->alpha = false;
+				current_checker->alpha = false;
 
-				add_obj_to_track_file(new_checker, tfl, FMT_RS(rs, vs), dump_tracking_rs);
+				add_obj_to_track_file(current_checker, tfl, FMT_RS(rs, vs), dump_tracking_rs);
 			}
 		}
 	}
