@@ -57,8 +57,7 @@ free_misc_check(checker_t *checker)
 {
 	misc_checker_t *misck_checker = checker->data;
 
-	if (misck_checker->script.args)
-		FREE(misck_checker->script.args);
+	notify_free_script(&misck_checker->script);
 	FREE(misck_checker);
 	FREE(checker);
 }
@@ -70,6 +69,8 @@ dump_misc_check(FILE *fp, const checker_t *checker)
 	char time_str[26];
 
 	conf_write(fp, "   Keepalive method = MISC_CHECK");
+	if (misck_checker->script.path)
+		conf_write(fp, "   path = %s", misck_checker->script.path);
 	conf_write(fp, "   script = %s", cmd_str(&misck_checker->script));
 	conf_write(fp, "   timeout = %lu", misck_checker->timeout/TIMER_HZ);
 	conf_write(fp, "   dynamic = %s", misck_checker->dynamic ? "YES" : "NO");
@@ -198,9 +199,6 @@ misc_end_handler(void)
 			return;
 		}
 	}
-
-	/* queue the checker */
-	list_add_tail(&current_checker->e_list, &checkers_queue);
 }
 
 void
@@ -223,36 +221,42 @@ install_misc_check_keyword(void)
 unsigned
 check_misc_script_security(magic_t magic)
 {
+	virtual_server_t *vs;
+	real_server_t *rs;
 	checker_t *checker, *checker_tmp;
 	misc_checker_t *misc_script;
 	unsigned script_flags = 0;
 	unsigned flags;
 	bool insecure;
 
-	list_for_each_entry_safe(checker, checker_tmp, &checkers_queue, e_list) {
-		if (checker->launch != misc_check_thread)
-			continue;
+	list_for_each_entry(vs, &check_data->vs, e_list) {
+		list_for_each_entry(rs, &vs->rs, e_list) {
+			list_for_each_entry_safe(checker, checker_tmp, &rs->checkers_list, rs_list) {
+				if (checker->launch != misc_check_thread)
+					continue;
 
-		misc_script = CHECKER_ARG(checker);
+				misc_script = CHECKER_ARG(checker);
 
-		script_flags |= (flags = check_script_secure(&misc_script->script, magic));
+				script_flags |= (flags = check_script_secure(&misc_script->script, magic));
 
-		/* Mark not to run if needs inhibiting */
-		insecure = false;
-		if (flags & SC_INHIBIT) {
-			log_message(LOG_INFO, "Disabling misc script %s due to insecure", cmd_str(&misc_script->script));
-			insecure = true;
-		}
-		else if (flags & SC_NOTFOUND) {
-			log_message(LOG_INFO, "Disabling misc script %s since not found/accessible", cmd_str(&misc_script->script));
-			insecure = true;
-		}
-		else if (!(flags & (SC_EXECUTABLE | SC_SYSTEM)))
-			insecure = true;
+				/* Mark not to run if needs inhibiting */
+				insecure = false;
+				if (flags & SC_INHIBIT) {
+					log_message(LOG_INFO, "Disabling misc script %s due to insecure", cmd_str(&misc_script->script));
+					insecure = true;
+				}
+				else if (flags & SC_NOTFOUND) {
+					log_message(LOG_INFO, "Disabling misc script %s since not found/accessible", cmd_str(&misc_script->script));
+					insecure = true;
+				}
+				else if (!(flags & (SC_EXECUTABLE | SC_SYSTEM)))
+					insecure = true;
 
-		if (insecure) {
-			/* Remove the script */
-			free_checker(checker);
+				if (insecure) {
+					/* Remove the script */
+					free_checker(checker);
+				}
+			}
 		}
 	}
 
@@ -334,9 +338,10 @@ misc_check_child_thread(thread_ref_t thread)
 					/* The process does not exist, and we should
 					 * have reaped its exit status, otherwise it
 					 * would exist as a zombie process. */
-					log_message(LOG_INFO, "Misc script %s child (PID %d) lost", misck_checker->script.args[0], pid);
+					log_message(LOG_INFO, "Misc script %s child (PID %d) lost, register checker again", misck_checker->script.args[0], pid);
 					misck_checker->state = SCRIPT_STATE_IDLE;
 					timeout = 0;
+					goto recheck;
 				} else {
 					log_message(LOG_INFO, "kill -%d of process %s(%d) with new state %u failed with errno %d", sig_num, misck_checker->script.args[0], pid, misck_checker->state, errno);
 					timeout = 1000;
@@ -485,6 +490,7 @@ misc_check_child_thread(thread_ref_t thread)
 		}
 	}
 
+recheck:
 	/* Register next timer checker */
 	next_time = timer_add_long(misck_checker->last_ran, checker->retry_it ? checker->delay_before_retry : checker->delay_loop);
 	next_time = timer_sub_now(next_time);
