@@ -339,6 +339,10 @@ free_parent_mallocs_exit(void)
 #endif
 
 	FREE_CONST_PTR(config_id);
+
+#ifdef _REPRODUCIBLE_BUILD_
+	FREE_CONST_PTR(config_opts);
+#endif
 }
 
 const char *
@@ -990,7 +994,7 @@ reload_check_child_thread(thread_ref_t thread)
 static void
 start_validate_reload_conf_child(void)
 {
-	notify_script_t script;
+	notify_script_t script = { .path = NULL };
 	int i;
 	int ret;
 	int argc;
@@ -1001,6 +1005,7 @@ start_validate_reload_conf_child(void)
 	int fd;
 	int len;
 	char exe_buf[128];
+	struct stat sb;
 
 	exe_buf[sizeof(exe_buf) - 1] = '\0';
 	ret = readlink("/proc/self/exe", exe_buf, sizeof(exe_buf));
@@ -1055,8 +1060,13 @@ start_validate_reload_conf_child(void)
 	script.uid = 0;
 	script.gid = 0;
 
-	if (truncate(global_data->reload_check_config, 0) && errno != ENOENT)
-		log_message(LOG_INFO, "truncate of config check log %s failed (%d) - %m", global_data->reload_check_config, errno);
+	if (truncate(global_data->reload_check_config, 0) && errno != ENOENT) {
+		/* The file exists, but truncate failed. It might be a character
+		 * device like /dev/null. truncate() returns EINVAL in this case. */
+		if (stat(global_data->reload_check_config, &sb) ||
+		    !S_ISCHR(sb.st_mode))
+			log_message(LOG_INFO, "truncate of config check log %s failed (%d) - %m", global_data->reload_check_config, errno);
+	}
 
 	create_reload_file();
 
@@ -1771,6 +1781,41 @@ report_distro(void)
 	fclose(fp);
 }
 
+#ifdef _REPRODUCIBLE_BUILD_
+static char *
+read_config_opts(const char *filename)
+{
+	struct stat statbuf;
+	int fd;
+	char *opts_buf;
+
+	if (stat(filename, &statbuf))
+		return NULL;
+
+	if ((fd = open(filename, O_RDONLY)) == -1) {
+		fprintf(stderr, "Failed to open %s\n", filename);
+		return NULL;
+	}
+
+	opts_buf = malloc(statbuf.st_size);
+
+	/* Read, skipping trailing \n */
+	if (read(fd, opts_buf, statbuf.st_size - 1) != statbuf.st_size - 1) {
+		fprintf(stderr, "Failed to read %s\n", filename);
+
+		close(fd);
+		free(opts_buf);
+
+		return NULL;
+	}
+
+	opts_buf[statbuf.st_size - 1] = '\0';
+	close(fd);
+
+	return opts_buf;
+}
+#endif
+
 /* Usage function */
 static void
 usage(const char *prog)
@@ -2071,7 +2116,7 @@ parse_cmdline(int argc, char **argv)
 			fprintf(stderr, "Running on %s %s %s\n", uname_buf.sysname, uname_buf.release, uname_buf.version);
 			report_distro();
 			fprintf(stderr, "\n");
-			fprintf(stderr, "configure options: %s\n\n", KEEPALIVED_CONFIGURE_OPTIONS);
+			fprintf(stderr, "configure options: %s\n\n", config_opts);
 			fprintf(stderr, "Config options: %s\n\n", CONFIGURATION_OPTIONS);
 			fprintf(stderr, "System options: %s\n", SYSTEM_OPTIONS);
 			exit(0);
@@ -2414,6 +2459,29 @@ keepalived_main(int argc, char **argv)
 	}
 #endif
 
+#ifdef _REPRODUCIBLE_BUILD_
+	char *config_opts_read;
+
+	if (!(config_opts_read = read_config_opts(CONFIG_OPTS_FILE_PRIMARY))) {
+		/* Look for the config-opts file in same location as executable */
+		const char *suffix = ".config-opts";
+		char *file = malloc(strlen(argv[0]) + strlen(suffix) + 1);
+
+		strcpy(file, argv[0]);
+		strcat(file, suffix);
+
+		config_opts_read = read_config_opts(file);
+		free(file);
+	}
+
+	if (!config_opts_read) {
+		fprintf(stderr, "Unable to read build config options file\n");
+		exit(1);
+	}
+
+	config_opts = config_opts_read;
+#endif
+
 #ifdef _MEM_CHECK_
 	__set_bit(MEM_CHECK_BIT, &debug);
 #endif
@@ -2515,6 +2583,13 @@ keepalived_main(int argc, char **argv)
 	/* Handle any core file requirements */
 	core_dump_init();
 
+#ifdef _REPRODUCIBLE_BUILD_
+	/* We want to use our MALLOC functions */
+	char *new_config_opts_str = STRDUP(config_opts);
+	free(config_opts_read);
+	config_opts = new_config_opts_str;
+#endif
+
 	if (os_major) {
 		if (KERNEL_VERSION(os_major, os_minor, os_release) < LINUX_VERSION_CODE) {
 			/* keepalived was built for a later kernel version */
@@ -2706,12 +2781,26 @@ keepalived_main(int argc, char **argv)
 	}
 
 	/* daemonize process */
-	if (!__test_bit(DONT_FORK_BIT, &debug) && xdaemon() > 0) {
-		closelog();
-		FREE_CONST_PTR(config_id);
-		FREE_PTR(orig_core_dump_pattern);
-		close_std_fd();
-		exit(0);
+	if (!__test_bit(DONT_FORK_BIT, &debug)) {
+		pid_t old_ppid = getpid();
+
+		if (xdaemon() > 0) {
+			/* Parent process */
+			closelog();
+			FREE_CONST_PTR(config_id);
+			FREE_PTR(orig_core_dump_pattern);
+			close_std_fd();
+			exit(0);
+		}
+
+		/* Child process */
+
+		/* check_start_stop_script_secure() called below makes a check
+		 * that our parent process hasn't changed, but it can take a while
+		 * for the parent of the fork to exit which is when our parent pid
+		 * changes. We need to loop until that has happened. */
+		while (old_ppid == getppid())
+			usleep(10);
 	}
 
 #ifdef _MEM_CHECK_
@@ -2719,6 +2808,9 @@ keepalived_main(int argc, char **argv)
 #endif
 
 	if (global_data->startup_script || global_data->shutdown_script) {
+		/* This is a workaround for the check in check_start_stop_script_secure() */
+		main_pid = getppid();
+
 		magic = ka_magic_open();
 		script_flags = 0;
 		if (global_data->startup_script)
@@ -2829,5 +2921,10 @@ end:
 #endif
 	close_std_fd();
 
+#ifdef _REPRODUCIBLE_BUILD_
+	FREE_CONST_PTR(config_opts);
+	config_opts = "removed";
+#endif
+
 	return exit_code;
 }
