@@ -30,6 +30,9 @@
 #include <sys/prctl.h>
 #include <sys/time.h>
 #include <fcntl.h>
+#ifdef _WITH_PROFILING_
+#include <sys/gmon.h>
+#endif
 
 #ifdef THREAD_DUMP
 #ifdef _WITH_SNMP_
@@ -755,8 +758,6 @@ sigusr2_vrrp(__attribute__((unused)) void *v, int sig)
 static void
 sigjson_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 {
-	log_message(LOG_INFO, "Printing VRRP as json for process(%d) on signal",
-		getpid());
 	thread_add_event(master, print_vrrp_json, NULL, 0);
 }
 #endif
@@ -1030,6 +1031,11 @@ start_vrrp_child(void)
 		return 0;
 	}
 
+#ifdef _WITH_PROFILING_
+	/* See https://lists.gnu.org/archive/html/bug-gnu-utils/2001-09/msg00047.html for details */
+	monstartup ((u_long) &_start, (u_long) &etext);
+#endif
+
 	prctl(PR_SET_PDEATHSIG, SIGTERM);
 
 	/* Check our parent hasn't already changed since the fork */
