@@ -83,6 +83,11 @@ bool do_epoll_thread_dump;
 #ifdef _SCRIPT_DEBUG_
 bool do_script_debug;
 #endif
+#ifndef _REPRODUCIBLE_BUILD_
+const char *config_opts = KEEPALIVED_CONFIGURE_OPTIONS;
+#else
+const char *config_opts = "not read";
+#endif
 
 /* local variables */
 static bool shutting_down;
@@ -593,7 +598,7 @@ report_child_status(int status, pid_t pid, char const *prog_name)
 			uname(&uname_buf);
 			log_message(LOG_INFO, "  Running on %s %s %s", uname_buf.sysname, uname_buf.release, uname_buf.version);
 			log_command_line(2);
-			log_options("configure options", KEEPALIVED_CONFIGURE_OPTIONS, 2);
+			log_options("configure options", config_opts, 2);
 			log_options("Config options", CONFIGURATION_OPTIONS, 2);
 			log_options("System options", SYSTEM_OPTIONS, 2);
 
@@ -1311,13 +1316,11 @@ thread_close_fd(thread_ref_t thread_cp)
 }
 
 /* Add timer event thread. */
-thread_ref_t
-thread_add_timer_uval(thread_master_t *m, thread_func_t func, void *arg, unsigned val, unsigned long timer)
+static thread_ref_t
+thread_add_timer_uval_sands(thread_master_t *m, thread_func_t func, void *arg, unsigned val, const timeval_t *sands)
 {
 	thread_t *thread;
 
-	assert(m != NULL);
-
 	thread = thread_new(m);
 	thread->type = THREAD_TIMER;
 	thread->master = m;
@@ -1325,13 +1328,7 @@ thread_add_timer_uval(thread_master_t *m, thread_func_t func, void *arg, unsigne
 	thread->arg = arg;
 	thread->u.uval = val;
 
-	/* Do we need jitter here? */
-	if (timer == TIMER_NEVER)
-		thread->sands.tv_sec = TIMER_DISABLED;
-	else {
-		set_time_now();
-		thread->sands = timer_add_long(time_now, timer);
-	}
+	thread->sands = *sands;
 
 	/* Sort by timeval. */
 	rb_add_cached(&thread->n, &m->timer, thread_timer_less);
@@ -1339,12 +1336,36 @@ thread_add_timer_uval(thread_master_t *m, thread_func_t func, void *arg, unsigne
 	return thread;
 }
 
+thread_ref_t
+thread_add_timer_uval(thread_master_t *m, thread_func_t func, void *arg, unsigned val, unsigned long timer)
+{
+	timeval_t sands;
+
+	assert(m != NULL);
+
+	/* Do we need jitter here? */
+	if (timer == TIMER_NEVER)
+		sands.tv_sec = TIMER_DISABLED;
+	else {
+		set_time_now();
+		sands = timer_add_long(time_now, timer);
+	}
+
+	return thread_add_timer_uval_sands(m, func, arg, val, &sands);
+}
+
 thread_ref_t
 thread_add_timer(thread_master_t *m, thread_func_t func, void *arg, unsigned long timer)
 {
 	return thread_add_timer_uval(m, func, arg, 0, timer);
 }
 
+thread_ref_t
+thread_add_timer_sands(thread_master_t *m, thread_func_t func, void *arg, const timeval_t *sands)
+{
+	return thread_add_timer_uval_sands(m, func, arg, 0, sands);
+}
+
 void
 thread_update_arg2(thread_ref_t thread_cp, const thread_arg2 *u)
 {
